# Code for the deterministic (Ordinary Differential Equations) simulation
# of the NOR1 cell in the XOR model.....................Angel Goni-Moreno

import scipy
from scipy import integrate
from scipy import copy
from scipy import optimize
import random

# The rates:
v = scipy.zeros((26),'d')
def rate_eqs(S):

	v[0] = K1*S[0]*S[2]
	v[1] = K_1*S[3]
	v[2] = K2*S[1]*S[2]
	v[3] = K_2*S[4]
	v[4] = K3*S[1]*S[3]
	v[5] = K_3*S[5]
	v[6] = K4*S[0]*S[4]
	v[7] = K_4*S[5]
	v[8] = K5*S[3]
	v[9] = K6*S[4]
	v[10] = K7*S[5]
	v[11] = K8*S[9]
        v[12] = K25*S[9]*S[11]
        v[13] = K26*S[9]*S[12]
        v[14] = K22*S[8]*S[6]
        v[15] = K_22*S[7]
        v[16] = K20*S[8]
        v[17] = K21*S[6]
        v[18] = K8*S[10]
        v[19] = K23*S[10]*S[11]
        v[20] = K24*S[10]*S[12]
        v[21] = K25*S[9]*S[11]
        v[22] = K11
        v[23] = K12
        v[24] = K9*S[0]
        v[25] = K10*S[1]

	return v

# The ODEs:
def diff_eqs(S,t):

	Sdot = scipy.zeros((13),'d')

	v = rate_eqs(S)

	# Differential equations
	Sdot[0] = v[22] - v[0] - v[6] - v[24]
	Sdot[1] = v[23] - v[2] - v[4] - v[25]
	Sdot[2] = - v[0] + v[1] - v[2] + v[3]
	Sdot[3] = v[0] - v[1] - v[4] + v[5]
	Sdot[4] = v[2] - v[3] - v[6] + v[7]
	Sdot[5] = v[4] - v[5] + v[6] - v[7]
	Sdot[6] = - v[14] + v[15]
	Sdot[7] = v[14] - v[15]
	Sdot[8] = v[8] + v[9] + v[10] - v[14] - v[15] - v[16]
	Sdot[9] = v[8] + v[9] + v[10] - v[11] - v[12] - v[13]
	Sdot[10] = v[17] - v[18] - v[19] - v[20]
	Sdot[11] = - v[19] + v[20] - v[12]
	Sdot[12] = v[19] - v[20] + v[12]

	return Sdot

# Parameter values:
K1 = 0.1; K_1 = 20.0; K2 = 0.1; K_2 = 20.0; K3 = 0.1; K_3 = 20.0; K4 = 0.1; K_4 = 20.0; K5 = 100.0; K6 = 100.0; K7 = 150.0; K8 = 0.05; K9 = 0.05; K10 = 0.05; K11 = 500; K12 = 500;
K20 = 0.05; K21 = 150.0; K22 = 0.1; K_22 = 10.0; K23 = 0.02e-3; K24 = 0.02e-3; K25 = 0.02e-3; K26 = 0.02e-3;
A = 500; B = 500; G = 1; G_A = 0; G_B = 0; G_AB = 0; G3 = 1; G3_X = 0; fimB = 0; fimE = 0; X = 0; pmd_F1 = 5; pmd_F2 = 0


# ODE calculation:
t_start = 0.0; t_end = 300.0; t_inc = 1
t_range = scipy.arange(t_start,t_end+t_inc,t_inc)
t_course = scipy.integrate.odeint(diff_eqs,[A, B, G, G_A, G_B, G_AB, G3, G3_X, X, fimE, fimB, pmd_F1, pmd_F2],t_range)

# File to store the numbers to plot (monitoring purposes)
f= open('data.txt', 'w')

for i in range(len(t_range)):
	p = t_range[i]
	f.write(str(p) + ' ')
	j=0
	while j<13:
		f.write(str(t_course[:,j][i]) + ' ')
		j = j+1
	f.write('\n')
f.close()



