# Code for the deterministic (Ordinary Differential Equations) simulation
# of the NOR2 cell in the XOR model.....................Angel Goni-Moreno

import scipy
from scipy import integrate
from scipy import copy
from scipy import optimize
import random

# The rates:
v_NOR2 = scipy.zeros((26),'d')
def rate_eqs_NOR2(S):


	v_NOR2[0] = K1*S[0]*S[2]
	v_NOR2[1] = K_1*S[3]
	v_NOR2[2] = K2*S[1]*S[2]
	v_NOR2[3] = K_2*S[4]
	v_NOR2[4] = K3*S[1]*S[3]
	v_NOR2[5] = K_3*S[5]
	v_NOR2[6] = K4*S[0]*S[4]
	v_NOR2[7] = K_4*S[5]
	v_NOR2[8] = K5*S[3]
	v_NOR2[9] = K6*S[4]
	v_NOR2[10] = K7*S[5]
	v_NOR2[11] = K8*S[6]
	v_NOR2[12] = K11
	v_NOR2[13] = K9*S[0]
	v_NOR2[14] = K12
	v_NOR2[15] = K10*S[1]
	v_NOR2[16] = K13*S[6]*S[7]
	v_NOR2[17] = K14*S[7]
	v_NOR2[18] = K15*S[9]*S[10]
	v_NOR2[19] = K16*S[11]
	v_NOR2[20] = K18*S[9]
	v_NOR2[21] = K17*S[11]
	v_NOR2[22] = K19*S[12]
        v_NOR2[23] = K27*S[6]*S[13]
        v_NOR2[24] = K28*S[14]
        v_NOR2[25] = K29*S[13]

	return v_NOR2

# The ODEs:
def diff_eqs_NOR2(S,t):

	Sdot = scipy.zeros((15),'d')
	v = rate_eqs_NOR2(S)

	# Differential equations
	Sdot[0] = v[12] - v[0] - v[6] - v[13]
	Sdot[1] = v[24] + v[25] - v[2] - v[4] - v[15]
	Sdot[2] = -v[0] + v[1] - v[2] + v[3]
	Sdot[3] = v[0] - v[1] - v[4] + v[5]
	Sdot[4] = v[2] - v[3] - v[6] + v[7]
	Sdot[5] = v[4] - v[5] + v[6] - v[7]
	Sdot[6] = v[8] + v[9] + v[10] - v[11] - v[16] - v[23]
	Sdot[7] = - v[16]
	Sdot[8] = v[16]
	Sdot[9] = v[17] - v[20] - v[18] + v[19]
	Sdot[10] = - v[18] + v[19]
	Sdot[11] = v[18] - v[19]
	Sdot[12] = v[21] - v[22]
        Sdot[13] = - v[23]
        Sdot[14] = v[23]

	return Sdot

# Parameter values:
K1 = 0.1; K_1 = 20.0; K2 = 0.1; K_2 = 20.0; K3 = 0.1; K_3 = 20.0; K4 = 0.1; K_4 = 20.0; K5 = 100.0; K6 = 100.0; K7 = 150.0; K8 = 0.05; K9 = 0.05; K10 = 0.05; K11 = 500; K12 = 0; K13 = 0.02e-3;
K14 = 100.0; K15 = 0.1; K16 = 20.0; K17 = 150.0; K18 = 0.05; K19 = 0.07;
K27 = 0.02e-3; K28 = 150; K29 = 150
A = 1000; B = 0; G = 1; G_A = 0; G_F1 = 0; G_AF1 = 0; G2 = 1; G2_F2 = 0
F1 = 0; Cre = 0; pmd_F2 = 5; pmd = 0; pmd_F1 = 0; F2 = 0; GFP = 0; pmd_F1_F2 =0

# ODE calculation:
t_start = 0.0; t_end = 300.0; t_inc = 1
t_range = scipy.arange(t_start,t_end+t_inc,t_inc)
t_course = scipy.integrate.odeint(diff_eqs_NOR2,[A, F1, G, G_A, G_F1, G_AF1, Cre, pmd_F2, pmd, F2, G2, G2_F2, GFP, pmd_F1_F2, pmd_F1],t_range)

# File to store the numbers to plot (monitoring purposes)
f= open('data.txt', 'w')

for i in range(len(t_range)):
	p = t_range[i]
	f.write(str(p) + ' ')
	j=0
	while j<14:
		f.write(str(t_course[:,j][i]) + ' ')
		j = j+1
	f.write('\n')
f.close()








